==> allensdk-2.14.0-py3-none-any.whl: allensdk/brain_observatory/behavior/data_objects/metadata/subject_metadata/subject_metadata.py <==
from datetime import datetime
from typing import Optional, List

import pytz
from pynwb import NWBFile

from allensdk.core import DataObject
from allensdk.brain_observatory.behavior.data_objects import BehaviorSessionId
from allensdk.core import \
    JsonReadableInterface, LimsReadableInterface, NwbReadableInterface
from allensdk.core import \
    JsonWritableInterface, NwbWritableInterface
from allensdk.brain_observatory.behavior.data_objects.metadata \
    .subject_metadata.age import \
    Age
from allensdk.brain_observatory.behavior.data_objects.metadata \
    .subject_metadata.driver_line import \
    DriverLine
from allensdk.brain_observatory.behavior.data_objects.metadata \
    .subject_metadata.full_genotype import \
    FullGenotype
from allensdk.brain_observatory.behavior.data_objects.metadata \
    .subject_metadata.mouse_id import \
    MouseId
from allensdk.brain_observatory.behavior.data_objects.metadata \
    .subject_metadata.reporter_line import \
    ReporterLine
from allensdk.brain_observatory.behavior.data_objects.metadata \
    .subject_metadata.sex import \
    Sex
from allensdk.brain_observatory.behavior.schemas import SubjectMetadataSchema
from allensdk.brain_observatory.nwb import load_pynwb_extension
from allensdk.core.auth_config import LIMS_DB_CREDENTIAL_MAP
from allensdk.internal.api import PostgresQueryMixin, db_connection_creator


class SubjectMetadata(DataObject, LimsReadableInterface, NwbReadableInterface,
                      NwbWritableInterface, JsonReadableInterface,
                      JsonWritableInterface):
    """Subject metadata"""

    def __init__(self,
                 sex: Sex,
                 age: Age,
                 reporter_line: ReporterLine,
                 full_genotype: FullGenotype,
                 driver_line: DriverLine,
                 mouse_id: MouseId,
                 death_on: Optional[datetime] = None):
        super().__init__(name='subject_metadata', value=None,
                         is_value_self=True)
        if death_on is not None and death_on.tzinfo is None:
            # Add UTC tzinfo if not already set
            death_on = pytz.utc.localize(death_on)
        self._sex = sex
        self._age = age
        self._reporter_line = reporter_line
        self._full_genotype = full_genotype
        self._driver_line = driver_line
        self._mouse_id = mouse_id
        self._death_on = death_on

    @classmethod
    def from_lims(
            cls,
            behavior_session_id: BehaviorSessionId,
            lims_db: PostgresQueryMixin
    ) -> "SubjectMetadata":
        sex = Sex.from_lims(behavior_session_id=behavior_session_id.value,
                            lims_db=lims_db)
        age = Age.from_lims(behavior_session_id=behavior_session_id.value,
                            lims_db=lims_db)
        reporter_line = ReporterLine.from_lims(
            behavior_session_id=behavior_session_id.value,
            lims_db=lims_db
        )
        full_genotype = FullGenotype.from_lims(
            behavior_session_id=behavior_session_id.value, lims_db=lims_db)
        driver_line = DriverLine.from_lims(
            behavior_session_id=behavior_session_id.value, lims_db=lims_db)
        mouse_id = MouseId.from_lims(
            behavior_session_id=behavior_session_id.value,
            lims_db=lims_db)
        death_on = cls._get_death_date_from_lims(
            mouse_id=mouse_id.value, lims_db=lims_db)
        return cls(
            sex=sex,
            age=age,
            full_genotype=full_genotype,
            driver_line=driver_line,
            mouse_id=mouse_id,
            reporter_line=reporter_line,
            death_on=death_on)

    @classmethod
    def from_json(cls, dict_repr: dict) -> "SubjectMetadata":
        sex = Sex.from_json(dict_repr=dict_repr)
        age = Age.from_json(dict_repr=dict_repr)
        reporter_line = ReporterLine.from_json(dict_repr=dict_repr)
        full_genotype = FullGenotype.from_json(dict_repr=dict_repr)
        driver_line = DriverLine.from_json(dict_repr=dict_repr)
        mouse_id = MouseId.from_json(dict_repr=dict_repr)
        death_on = cls._get_death_date_from_lims(
            mouse_id=mouse_id.value,
            lims_db=db_connection_creator(
                fallback_credentials=LIMS_DB_CREDENTIAL_MAP))

        return cls(
            sex=sex,
            age=age,
            full_genotype=full_genotype,
            driver_line=driver_line,
            mouse_id=mouse_id,
            reporter_line=reporter_line,
            death_on=death_on
        )

    def to_json(self) -> dict:
        pass

    @classmethod
    def from_nwb(cls, nwbfile: NWBFile) -> "SubjectMetadata":
        mouse_id = MouseId.from_nwb(nwbfile=nwbfile)
        sex = Sex.from_nwb(nwbfile=nwbfile)
        age = Age.from_nwb(nwbfile=nwbfile)
        reporter_line = ReporterLine.from_nwb(nwbfile=nwbfile)
        driver_line = DriverLine.from_nwb(nwbfile=nwbfile)
        genotype = FullGenotype.from_nwb(nwbfile=nwbfile)

        return cls(
            mouse_id=mouse_id,
            sex=sex,
            age=age,
            reporter_line=reporter_line,
            driver_line=driver_line,
            full_genotype=genotype
        )

    def to_nwb(self, nwbfile: NWBFile) -> NWBFile:
        BehaviorSubject = load_pynwb_extension(SubjectMetadataSchema,
                                               'ndx-aibs-behavior-ophys')
        nwb_subject = BehaviorSubject(
            description="A visual behavior subject with a LabTracks ID",
            age=Age.to_iso8601(age=self.age_in_days),
            driver_line=self.driver_line,
            genotype=self.full_genotype,
            subject_id=str(self.mouse_id),
            reporter_line=str(self.reporter_line),
            sex=self.sex,
            species='Mus musculus')
        nwbfile.subject = nwb_subject
        return nwbfile

    @property
    def sex(self) -> str:
        return self._sex.value

    @property
    def age_in_days(self) -> Optional[int]:
        return self._age.value

    @property
    def reporter_line(self) -> Optional[str]:
        return self._reporter_line.value

    @property
    def full_genotype(self) -> str:
        return self._full_genotype.value

    @property
    def cre_line(self) -> Optional[str]:
        return self._full_genotype.parse_cre_line(warn=True)

    @property
    def driver_line(self) -> List[str]:
        return self._driver_line.value

    @property
    def mouse_id(self) -> int:
        return self._mouse_id.value

    def get_death_date(self) -> Optional[datetime]:
        """

        Notes
        -----
        Optional since we don't store this field when reading from NWB
        """
        return self._death_on

    @classmethod
    def _get_death_date_from_lims(
            cls,
            mouse_id: int,
            lims_db: PostgresQueryMixin
    ):
        query = f"""
            SELECT death_on
            FROM donors
            WHERE external_donor_name = '{mouse_id}'
        """
        res = lims_db.fetchall(query)
        res = res[0]
        if res is not None:
            # convert to datetime.datetime
            res = res.astype('datetime64[s]').astype(datetime)

            res = pytz.utc.localize(res)
        return res

==> allensdk-2.16.1-py3-none-any.whl: allensdk/brain_observatory/behavior/data_objects/metadata/subject_metadata/subject_metadata.py <==
from datetime import datetime
from typing import Optional, List

import pytz
from pynwb import NWBFile

from allensdk.core import DataObject
from allensdk.brain_observatory.behavior.data_objects import BehaviorSessionId
from allensdk.core import \
    JsonReadableInterface, LimsReadableInterface, NwbReadableInterface
from allensdk.core import NwbWritableInterface
from allensdk.brain_observatory.behavior.data_objects.metadata \
    .subject_metadata.age import \
    Age
from allensdk.brain_observatory.behavior.data_objects.metadata \
    .subject_metadata.driver_line import \
    DriverLine
from allensdk.brain_observatory.behavior.data_objects.metadata \
    .subject_metadata.full_genotype import \
    FullGenotype
from allensdk.brain_observatory.behavior.data_objects.metadata \
    .subject_metadata.mouse_id import \
    MouseId
from allensdk.brain_observatory.behavior.data_objects.metadata \
    .subject_metadata.reporter_line import \
    ReporterLine
from allensdk.brain_observatory.behavior.data_objects.metadata \
    .subject_metadata.sex import \
    Sex
from allensdk.brain_observatory.behavior.schemas import SubjectMetadataSchema
from allensdk.brain_observatory.nwb import load_pynwb_extension
from allensdk.core.auth_config import LIMS_DB_CREDENTIAL_MAP
from allensdk.internal.api import PostgresQueryMixin, db_connection_creator


class SubjectMetadata(DataObject, LimsReadableInterface, NwbReadableInterface,
                      NwbWritableInterface, JsonReadableInterface,
                      ):
    """Subject metadata"""

    def __init__(self,
                 sex: Sex,
                 age: Age,
                 reporter_line: ReporterLine,
                 full_genotype: FullGenotype,
                 driver_line: DriverLine,
                 mouse_id: MouseId,
                 death_on: Optional[datetime] = None):
        super().__init__(name='subject_metadata', value=None,
                         is_value_self=True)
        if death_on is not None and death_on.tzinfo is None:
            # Add UTC tzinfo if not already set
            death_on = pytz.utc.localize(death_on)
        self._sex = sex
        self._age = age
        self._reporter_line = reporter_line
        self._full_genotype = full_genotype
        self._driver_line = driver_line
        self._mouse_id = mouse_id
        self._death_on = death_on

    @classmethod
    def from_lims(
            cls,
            behavior_session_id: BehaviorSessionId,
            lims_db: PostgresQueryMixin
    ) -> "SubjectMetadata":
        sex = Sex.from_lims(behavior_session_id=behavior_session_id.value,
                            lims_db=lims_db)
        age = Age.from_lims(behavior_session_id=behavior_session_id.value,
                            lims_db=lims_db)
        reporter_line = ReporterLine.from_lims(
            behavior_session_id=behavior_session_id.value,
            lims_db=lims_db
        )
        full_genotype = FullGenotype.from_lims(
            behavior_session_id=behavior_session_id.value, lims_db=lims_db)
        driver_line = DriverLine.from_lims(
            behavior_session_id=behavior_session_id.value, lims_db=lims_db)
        mouse_id = MouseId.from_lims(
            behavior_session_id=behavior_session_id.value,
            lims_db=lims_db)
        death_on = cls._get_death_date_from_lims(
            mouse_id=mouse_id.value, lims_db=lims_db)
        return cls(
            sex=sex,
            age=age,
            full_genotype=full_genotype,
            driver_line=driver_line,
            mouse_id=mouse_id,
            reporter_line=reporter_line,
            death_on=death_on)

    @classmethod
    def from_json(cls, dict_repr: dict) -> "SubjectMetadata":
        sex = Sex.from_json(dict_repr=dict_repr)
        age = Age.from_json(dict_repr=dict_repr)
        reporter_line = ReporterLine.from_json(dict_repr=dict_repr)
        full_genotype = FullGenotype.from_json(dict_repr=dict_repr)
        driver_line = DriverLine.from_json(dict_repr=dict_repr)
        mouse_id = MouseId.from_json(dict_repr=dict_repr)
        death_on = cls._get_death_date_from_lims(
            mouse_id=mouse_id.value,
            lims_db=db_connection_creator(
                fallback_credentials=LIMS_DB_CREDENTIAL_MAP))

        return cls(
            sex=sex,
            age=age,
            full_genotype=full_genotype,
            driver_line=driver_line,
            mouse_id=mouse_id,
            reporter_line=reporter_line,
            death_on=death_on
        )

    @classmethod
    def from_nwb(cls, nwbfile: NWBFile) -> "SubjectMetadata":
        mouse_id = MouseId.from_nwb(nwbfile=nwbfile)
        sex = Sex.from_nwb(nwbfile=nwbfile)
        age = Age.from_nwb(nwbfile=nwbfile)
        reporter_line = ReporterLine.from_nwb(nwbfile=nwbfile)
        driver_line = DriverLine.from_nwb(nwbfile=nwbfile)
        genotype = FullGenotype.from_nwb(nwbfile=nwbfile)

        return cls(
            mouse_id=mouse_id,
            sex=sex,
            age=age,
            reporter_line=reporter_line,
            driver_line=driver_line,
            full_genotype=genotype
        )

    def to_nwb(self, nwbfile: NWBFile) -> NWBFile:
        BehaviorSubject = load_pynwb_extension(SubjectMetadataSchema,
                                               'ndx-aibs-behavior-ophys')
        nwb_subject = BehaviorSubject(
            description="A visual behavior subject with a LabTracks ID",
            age=Age.to_iso8601(age=self.age_in_days),
            driver_line=self.driver_line,
            genotype=self.full_genotype,
            subject_id=str(self.mouse_id),
            reporter_line=str(self.reporter_line),
            sex=self.sex,
            species='Mus musculus')
        nwbfile.subject = nwb_subject
        return nwbfile

    @property
    def sex(self) -> str:
        return self._sex.value

    @property
    def age_in_days(self) -> Optional[int]:
        return self._age.value

    @property
    def reporter_line(self) -> Optional[str]:
        return self._reporter_line.value

    @property
    def full_genotype(self) -> str:
        return self._full_genotype.value

    @property
    def cre_line(self) -> Optional[str]:
        return self._full_genotype.parse_cre_line(warn=True)

    @property
    def driver_line(self) -> List[str]:
        return self._driver_line.value

    @property
    def mouse_id(self) -> int:
        return self._mouse_id.value

    def get_death_date(self) -> Optional[datetime]:
        """

        Notes
        -----
        Optional since we don't store this field when reading from NWB
        """
        return self._death_on

    @classmethod
    def _get_death_date_from_lims(
            cls,
            mouse_id: int,
            lims_db: PostgresQueryMixin
    ):
        query = f"""
            SELECT death_on
            FROM donors
            WHERE external_donor_name = '{mouse_id}'
        """
        res = lims_db.fetchall(query)
        res = res[0]
        if res is not None:
            # convert to datetime.datetime
            res = res.astype('datetime64[s]').astype(datetime)

            res = pytz.utc.localize(res)
        return res
